/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.rest.beans.element;

import java.io.Serializable;
import java.util.UUID;

/**
 * Bean (data transfer object) for communication and (json, xml) serialization.
 * Intended for operations that modify names.
 *
 * @author Lars Johansson
 */
public class NameElementCommand implements Serializable {

    /**
     *
     */
    private static final long serialVersionUID = -3166840982436164399L;

    private UUID uuid;
    private UUID parentsystemstructure;
    private UUID parentdevicestructure;
    private String index;
    private String description;
    private String comment;

    /**
     * Public constructor.
     */
    public NameElementCommand() {
    }

    /**
     * Public constructor.
     *
     * @param uuid uuid
     * @param parentsystemstructure parent system structure uuid
     * @param parentdevicestructure parent device structure uuid
     * @param index index
     * @param description description
     * @param comment comment
     */
    public NameElementCommand(
            UUID uuid, UUID parentsystemstructure, UUID parentdevicestructure, String index,
            String description, String comment) {
        super();
        this.uuid = uuid;
        this.parentsystemstructure = parentsystemstructure;
        this.parentdevicestructure = parentdevicestructure;
        this.index = index;
        this.description = description;
        this.comment = comment;
    }

    public UUID getUuid() {
        return uuid;
    }
    public void setUuid(UUID uuid) {
        this.uuid = uuid;
    }
    public UUID getParentsystemstructure() {
        return parentsystemstructure;
    }
    public void setParentsystemstructure(UUID parentsystemstructure) {
        this.parentsystemstructure = parentsystemstructure;
    }
    public UUID getParentdevicestructure() {
        return parentdevicestructure;
    }
    public void setParentdevicestructure(UUID parentdevicestructure) {
        this.parentdevicestructure = parentdevicestructure;
    }
    public String getIndex() {
        return index;
    }
    public void setIndex(String index) {
        this.index = index;
    }
    public String getDescription() {
        return description;
    }
    public void setDescription(String description) {
        this.description = description;
    }
    public String getComment() {
        return comment;
    }
    public void setComment(String comment) {
        this.comment = comment;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;

        return equals ((NameElementCommand) obj);
    }

    public boolean equals(NameElementCommand other) {
        if (other == null)
            return false;

        if (!super.equals(other)) {
            return false;
        }

        if (getUuid() == null) {
            if (other.getUuid() != null)
                return false;
        } else if (!getUuid().equals(other.getUuid()))
            return false;
        if (getParentsystemstructure() == null) {
            if (other.getParentsystemstructure() != null)
                return false;
        } else if (!getParentsystemstructure().equals(other.getParentsystemstructure()))
            return false;
        if (getParentdevicestructure() == null) {
            if (other.getParentdevicestructure() != null)
                return false;
        } else if (!getParentdevicestructure().equals(other.getParentdevicestructure()))
            return false;
        if (getIndex() == null) {
            if (other.getIndex() != null)
                return false;
        } else if (!getIndex().equals(other.getIndex()))
            return false;
        if (getDescription() == null) {
            if (other.getDescription() != null)
                return false;
        } else if (!getDescription().equals(other.getDescription()))
            return false;
        if (getComment() == null) {
            if (other.getComment() != null)
                return false;
        } else if (!getComment().equals(other.getComment()))
            return false;

        return true;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        sb.append("\"uuid\": "                    + getUuid());
        sb.append(", \"parentsystemstructure\": " + getParentsystemstructure());
        sb.append(", \"parentdevicestructure\": " + getParentdevicestructure());
        sb.append(", \"index\": "                 + getIndex());
        sb.append(", \"description\": "           + getDescription());
        sb.append(", \"comment\": "               + getComment());
        sb.append("}");
        return sb.toString();
    }

}
