/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.rest.beans.element;

import java.io.Serializable;
import java.util.Date;
import java.util.UUID;

import org.openepics.names.rest.beans.Status;
import org.openepics.names.rest.beans.Type;

/**
 * Bean (data transfer object) for communication and (json, xml) serialization.
 * Intended for operations that read structures.
 *
 * @author Lars Johansson
 */
public class StructureElement extends StructureElementCommand implements Serializable {

    /**
     *
     */
    private static final long serialVersionUID = 356654987707139728L;

    private String mnemonicpath;
    private Integer level;
    private Status status;
    private Boolean latest;
    private Boolean deleted;
    private Date when;
    private String who;

    /**
     * Public constructor.
     */
    public StructureElement() {
        super();
    }

    /**
     * Public constructor.
     *
     * @param uuid uuid
     * @param type type
     * @param parent parent
     * @param name name
     * @param mnemonic mnemonic
     * @param description description
     * @param comment comment
     */
    public StructureElement(
            UUID uuid, Type type, UUID parent,
            String name, String mnemonic,
            String description, String comment) {
        super(uuid, type, parent, name, mnemonic, description, comment);
    }

    /**
     * Public constructor.
     *
     * @param uuid uuid
     * @param type type
     * @param parent parent
     * @param name name
     * @param mnemonic mnemonic
     * @param description description
     * @param comment comment
     * @param mnemonicpath mnemonic path
     * @param level level
     * @param status status
     * @param latest latest
     * @param deleted deleted
     * @param when when
     * @param who who
     */
    public StructureElement(
            UUID uuid, Type type, UUID parent, String name, String mnemonic,
            String description, String comment,
            String mnemonicpath, Integer level,
            Status status, Boolean latest, Boolean deleted,
            Date when, String who) {
        super(uuid, type, parent, name, mnemonic, description, comment);
        this.mnemonicpath = mnemonicpath;
        this.level = level;
        this.status = status;
        this.latest = latest;
        this.deleted = deleted;
        this.when = when;
        this.who = who;
    }

    public String getMnemonicpath() {
        return mnemonicpath;
    }
    public void setMnemonicpath(String mnemonicpath) {
        this.mnemonicpath = mnemonicpath;
    }
    public Integer getLevel() {
        return level;
    }
    public void setLevel(Integer level) {
        this.level = level;
    }
    public Status getStatus() {
        return status;
    }
    public void setStatus(Status status) {
        this.status = status;
    }
    public Boolean isLatest() {
        return latest;
    }
    public void setLatest(Boolean latest) {
        this.latest = latest;
    }
    public Boolean isDeleted() {
        return deleted;
    }
    public void setDeleted(Boolean deleted) {
        this.deleted = deleted;
    }
    public Date getWhen() {
        return when;
    }
    public void setWhen(Date when) {
        this.when = when;
    }
    public String getWho() {
        return who;
    }
    public void setWho(String who) {
        this.who = who;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;

        return equals ((StructureElement) obj);
    }

    public boolean equals(StructureElement other) {
        if (other == null)
            return false;

        if (!super.equals(other)) {
            return false;
        }

        if (getMnemonicpath() == null) {
            if (other.getMnemonicpath() != null)
                return false;
        } else if (!getMnemonicpath().equals(other.getMnemonicpath()))
            return false;
        if (getLevel() == null) {
            if (other.getLevel() != null)
                return false;
        } else if (!getLevel().equals(other.getLevel()))
            return false;
        if (getStatus() == null) {
            if (other.getStatus() != null)
                return false;
        } else if (!getStatus().equals(other.getStatus()))
            return false;
        if (isLatest() == null) {
            if (other.isLatest() != null)
                return false;
        } else if (!isLatest().equals(other.isLatest()))
            return false;
        if (isDeleted() == null) {
            if (other.isDeleted() != null)
                return false;
        } else if (!isDeleted().equals(other.isDeleted()))
            return false;
        if (getWhen() == null) {
            if (other.getWhen() != null)
                return false;
        } else if (!getWhen().equals(other.getWhen()))
            return false;
        if (getWho() == null) {
            if (other.getWho() != null)
                return false;
        } else if (!getWho().equals(other.getWho()))
            return false;

        return true;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        sb.append("\"uuid\": "           + getUuid());
        sb.append(", \"type\": "         + getType());
        sb.append(", \"parent_uuid\": "  + getParent());
        sb.append(", \"name\": "         + getName());
        sb.append(", \"mnemonic\": "     + getMnemonic());
        sb.append(", \"mnemonicpath\": " + getMnemonicpath());
        sb.append(", \"level\": "        + getLevel());
        sb.append(", \"description\": "  + getDescription());
        sb.append(", \"status\": "       + getStatus());
        sb.append(", \"latest\": "       + isLatest());
        sb.append(", \"deleted\": "      + isDeleted());
        sb.append(", \"when\": "         + getWhen());
        sb.append(", \"who\": "          + getWho());
        sb.append(", \"comment\": "      + getComment());
        sb.append("}");
        return sb.toString();
    }

    public String toStringSimple() {
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        sb.append("\"uuid\": "           + getUuid());
        sb.append(", \"type\": "         + getType());
        sb.append(", \"mnemonic\": "     + getMnemonic());
        sb.append(", \"status\": "       + getStatus());
        sb.append(", \"latest\": "       + isLatest());
        sb.append(", \"deleted\": "      + isDeleted());
        sb.append("}");
        return sb.toString();
    }

}
