/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.rest.beans.element;

import java.io.Serializable;
import java.util.UUID;

import org.openepics.names.rest.beans.Type;

/**
 * Bean (data transfer object) for communication and (json, xml) serialization.
 * Intended for operations that modify structures.
 *
 * @author Lars Johansson
 */
public class StructureElementCommand implements Serializable {

    /**
     *
     */
    private static final long serialVersionUID = 788979309267676232L;

    private UUID uuid;
    private Type type;
    private UUID parent;
    private String name;
    private String mnemonic;
    private String description;
    private String comment;

    /**
     * Public constructor.
     */
    public StructureElementCommand() {
    };

    /**
     * Public constructor.
     *
     * @param uuid uuid
     * @param type type
     * @param parent parent
     * @param name name
     * @param mnemonic mnemonic
     * @param description description
     * @param comment comment
     */
    public StructureElementCommand(
            UUID uuid, Type type, UUID parent,
            String name, String mnemonic,
            String description, String comment) {
        super();
        this.uuid = uuid;
        this.type = type;
        this.parent = parent;
        this.name = name;
        this.mnemonic = mnemonic;
        this.description = description;
        this.comment = comment;
    }

    public UUID getUuid() {
        return uuid;
    }
    public void setUuid(UUID uuid) {
        this.uuid = uuid;
    }
    public Type getType() {
        return type;
    }
    public void setType(Type type) {
        this.type = type;
    }
    public UUID getParent() {
        return parent;
    }
    public void setParent(UUID parent) {
        this.parent = parent;
    }
    public String getName() {
        return name;
    }
    public void setName(String name) {
        this.name = name;
    }
    public String getMnemonic() {
        return mnemonic;
    }
    public void setMnemonic(String mnemonic) {
        this.mnemonic = mnemonic;
    }
    public String getDescription() {
        return description;
    }
    public void setDescription(String description) {
        this.description = description;
    }
    public String getComment() {
        return comment;
    }
    public void setComment(String comment) {
        this.comment = comment;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;

        return equals((StructureElementCommand) obj);
    }

    public boolean equals(StructureElementCommand other) {
        if (other == null)
            return false;

        if (getUuid() == null) {
            if (other.getUuid() != null)
                return false;
        } else if (!getUuid().equals(other.getUuid()))
            return false;
        if (getType() == null) {
            if (other.getType() != null)
                return false;
        } else if (!getType().equals(other.getType()))
            return false;
        if (getParent() == null) {
            if (other.getParent() != null)
                return false;
        } else if (!getParent().equals(other.getParent()))
            return false;
        if (getName() == null) {
            if (other.getName() != null)
                return false;
        } else if (!getName().equals(other.getName()))
            return false;
        if (getMnemonic() == null) {
            if (other.getMnemonic() != null)
                return false;
        } else if (!getMnemonic().equals(other.getMnemonic()))
            return false;
        if (getDescription() == null) {
            if (other.getDescription() != null)
                return false;
        } else if (!getDescription().equals(other.getDescription()))
            return false;
        if (getComment() == null) {
            if (other.getComment() != null)
                return false;
        } else if (!getComment().equals(other.getComment()))
            return false;

        return true;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        sb.append("\"uuid\": "           + getUuid());
        sb.append(", \"type\": "         + getType());
        sb.append(", \"parent_uuid\": "  + getParent());
        sb.append(", \"name\": "         + getName());
        sb.append(", \"mnemonic\": "     + getMnemonic());
        sb.append(", \"description\": "  + getDescription());
        sb.append(", \"comment\": "      + getComment());
        sb.append("}");
        return sb.toString();
    }

}
